@@ -23,6 +23,7 @@ from calculate.lib.utils.text import _u8
 
 from M2Crypto import m2
 from M2Crypto.X509 import X509_Extension
+from calculate.lib.utils.files import writeFile, readFile
 from ctypes import *
  
 
@@ -110,57 +111,79 @@ class CreateCertError(Exception):
     pass
 
 def create_selfsigned_ca(dn_data, keyfile, certfile):
-    from calculate.lib.utils.gpg import Pipe
-    from calculate.lib.utils.files import process
-    from string import Template
-    ca_conf = Template("""
-[ req ]
-default_bits = 2048
-distinguished_name = req_distinguished_name
-prompt = no
-x509_extensions = v3_ca
+    from OpenSSL import crypto
 
-[ req_distinguished_name ]
-CN = $CN
-ST = $ST
-L = $L
-O = $O
-OU = $OU
+    certpem = readFile(keyfile)
+    if not certpem:
+        raise CreateCertError(_("Key file {} not found").format(keyfile))
 
-[ v3_ca ]
-subjectKeyIdentifier=hash
-authorityKeyIdentifier=keyid:always,issuer
-basicConstraints = critical,CA:true
-""")
-    with Pipe() as cfgpipe:
-        p = process("openssl", "req", "-x509", "-new", "-key", keyfile,
-                    "-config", cfgpipe.get_filename(), "-out", certfile,
-                    "-days", "7300")
-        cfgpipe.write(ca_conf.safe_substitute(dn_data))
-        cfgpipe.closein()
-        if p.failed():
-            raise CreateCertError(p.readerr())
+    try:
+        pkey = crypto.load_privatekey(
+            crypto.FILETYPE_PEM, certpem)
+        ca = crypto.X509()
+        ca.set_version(2)
+        subject = ca.get_subject()
+        subject.countryName = dn_data['C']
+        subject.commonName = dn_data['CN']
+        subject.stateOrProvinceName = dn_data['ST']
+        subject.localityName = dn_data['L']
+        subject.organizationName = dn_data['O']
+        subject.organizationalUnitName = dn_data['OU']
+
+        ca.gmtime_adj_notBefore(-60*60*24)
+        ca.gmtime_adj_notAfter(60*60*24*365*20)
+        ca.set_issuer(subject)
+        ca.set_pubkey(pkey)
+        ca.add_extensions([
+            crypto.X509Extension(b'basicConstraints', True, b'CA:TRUE'),
+            #crypto.X509Extension(b'keyUsage', False, b'keyCertSign, cRLSign'),
+            crypto.X509Extension(b'subjectKeyIdentifier', False, b'hash', subject=ca)])
+        ca.add_extensions([crypto.X509Extension(b'authorityKeyIdentifier', False, b'keyid:always',issuer=ca)])
+        ca.sign(pkey, 'sha1')
+
+        with writeFile(certfile) as f:
+            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, ca))
+    except crypto.Error as e:
+        raise CreateCertError(str(e))
 
 def sign_client_certifacation_request(ca_keyfile, ca_certfile, requestfile, out_cert, group):
-    from calculate.lib.utils.gpg import Pipe
-    from calculate.lib.utils.files import process
-    from string import Template
-    client_conf = Template("""
-[ client ]
-nsCertType = client
-keyUsage = digitalSignature, keyEncipherment
-extendedKeyUsage = clientAuth
-nsComment = group:$group
-""")
-    with Pipe() as cfgpipe:
-        p = process("openssl", "x509", "-req", "-in", requestfile,
-                    "-CA", ca_certfile,
-                    "-CAkey", ca_keyfile, "-CAcreateserial",
-                    "-extfile", cfgpipe.get_filename(),
-                    "-extensions", "client",
-                    "-out", out_cert,
-                    "-days", "7300")
-        cfgpipe.write(client_conf.safe_substitute({"group":group}))
-        cfgpipe.closein()
-        if p.failed():
-            raise CreateCertError(p.readerr())
+    from OpenSSL import crypto
+    cakeyfilepem = readFile(ca_keyfile)
+    cacertpem = readFile(ca_certfile)
+    requestpem = readFile(requestfile)
+    if not cakeyfilepem:
+        raise CreateCertError(
+            _("Key file {} not found").format(ca_keyfile))
+    if not cacertpem:
+        raise CreateCertError(
+            _("CA certitficate file {} not found").format(ca_certfile))
+    if not requestpem:
+        raise CreateCertError(
+            _("Request file {} not found").format(requestfile))
+
+    try:
+        pkey = crypto.load_privatekey(crypto.FILETYPE_PEM, cakeyfilepem)
+        ca = crypto.load_certificate(crypto.FILETYPE_PEM, cacertpem)
+        req = crypto.load_certificate_request(crypto.FILETYPE_PEM, requestpem)
+
+        cert = crypto.X509()
+        cert.set_version(2)
+        cert.gmtime_adj_notBefore(-60*60*24)
+        cert.gmtime_adj_notAfter(60*60*24*365*20)
+        cert.set_issuer(ca.get_subject())
+        cert.set_subject(req.get_subject())
+        cert.set_pubkey(req.get_pubkey())
+
+        cert.add_extensions([
+            crypto.X509Extension(b'basicConstraints', False, b'CA:FALSE'),
+            crypto.X509Extension(b'nsCertType', False, b'client'),
+            crypto.X509Extension(b'keyUsage', False, b'digitalSignature, keyEncipherment'),
+            crypto.X509Extension(b'extendedKeyUsage', False, b'clientAuth'),
+            crypto.X509Extension(b'nsComment', False, 'group:{}'.format(group)),
+        ])
+        cert.sign(pkey, 'sha1')
+
+        with writeFile(out_cert) as f:
+            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
+    except crypto.Error as e:
+        raise CreateCertError(str(e))
